Package: BCRANK
Title: Predicting binding site consensus from ranked DNA sequences
Version: 1.74.0
Date: 2010-08-25
Depends: methods
Author: Adam Ameur <Adam.Ameur@genpat.uu.se>
Description: Functions and classes for de novo prediction of transcription factor binding consensus by heuristic search
Maintainer: Adam Ameur <Adam.Ameur@genpat.uu.se>
License: GPL-2
LazyLoad: true
Collate: AllClasses.R AllGenerics.R methods-BCRANKmatch.R
        methods-BCRANKsearch.R methods-BCRANKresult.R BCRANK.R
Imports: Biostrings
Suggests: seqLogo
biocViews: MotifDiscovery, GeneRegulation
git_url: https://git.bioconductor.org/packages/BCRANK
git_branch: RELEASE_3_23
git_last_commit: ef18c99
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:47:28 UTC; biocbuild
